@@ -12,11 +12,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import copy
 import logging
 
 import jsonschema
 import metalsmith
+from metalsmith import instance_config
 from metalsmith import sources
 from mistral_lib import actions
 from openstack import exceptions as sdk_exc
@@ -33,39 +33,59 @@ def _provisioner(context):
     return metalsmith.Provisioner(session=session)
 
 
+_IMAGE_SCHEMA = {
+    'type': 'object',
+    'properties': {
+        'href': {'type': 'string'},
+        'checksum': {'type': 'string'},
+        'kernel': {'type': 'string'},
+        'ramdisk': {'type': 'string'},
+    },
+    'required': ['href'],
+    'additionalProperties': False,
+}
+
+_NIC_SCHEMA = {
+    'type': 'object',
+    'properties': {
+        'network': {'type': 'string'},
+        'port': {'type': 'string'},
+        'fixed_ip': {'type': 'string'},
+        'subnet': {'type': 'string'},
+    },
+    'additionalProperties': False
+}
+
 _INSTANCE_SCHEMA = {
     'type': 'object',
     'properties': {
         'capabilities': {'type': 'object'},
-        'hostname': {'type': 'string',
-                     'minLength': 2,
-                     'maxLength': 255},
-        'image': {'type': 'string'},
-        'image_checksum': {'type': 'string'},
-        'image_kernel': {'type': 'string'},
-        'image_ramdisk': {'type': 'string'},
+        'hostname': {
+            'type': 'string',
+            'minLength': 2,
+            'maxLength': 255
+        },
+        'image': _IMAGE_SCHEMA,
         'name': {'type': 'string'},
-        'nics': {'type': 'array',
-                 'items': {'type': 'object',
-                           'properties': {
-                               'network': {'type': 'string'},
-                               'port': {'type': 'string'},
-                               'fixed_ip': {'type': 'string'},
-                               'subnet': {'type': 'string'},
-                           },
-                           'additionalProperties': False}},
+        'nics': {
+            'type': 'array',
+            'items': _NIC_SCHEMA
+        },
         'profile': {'type': 'string'},
+        'provisioned': {'type': 'boolean'},
         'resource_class': {'type': 'string'},
         'root_size_gb': {'type': 'integer', 'minimum': 4},
         'swap_size_mb': {'type': 'integer', 'minimum': 64},
-        'traits': {'type': 'array',
-                   'items': {'type': 'string'}},
+        'traits': {
+            'type': 'array',
+            'items': {'type': 'string'}
+        },
     },
     'additionalProperties': False,
 }
 
 
-_INSTANCES_INPUT_SCHEMA = {
+_INSTANCES_SCHEMA = {
     'type': 'array',
     'items': _INSTANCE_SCHEMA
 }
@@ -78,11 +98,10 @@ _ROLES_INPUT_SCHEMA = {
         'type': 'object',
         'properties': {
             'name': {'type': 'string'},
-            'count': {'type': 'integer', 'minimum': 0},
-            'profile': {'type': 'string'},
             'hostname_format': {'type': 'string'},
-            'instances': {'type': 'array',
-                          'items': _INSTANCE_SCHEMA},
+            'count': {'type': 'integer', 'minimum': 0},
+            'defaults': _INSTANCE_SCHEMA,
+            'instances': _INSTANCES_SCHEMA,
         },
         'additionalProperties': False,
         'required': ['name'],
@@ -216,7 +235,6 @@ class DeployNodeAction(base.TripleOAction):
     def __init__(self, instance, node, ssh_keys=None,
                  # For compatibility with deployment based on heat+nova
                  ssh_user_name='heat-admin',
-                 default_image='overcloud-full',
                  default_network='ctlplane',
                  # 50 is the default for old flavors, subtracting 1G to account
                  # for partitioning and configdrive.
@@ -224,16 +242,14 @@ class DeployNodeAction(base.TripleOAction):
         super(DeployNodeAction, self).__init__()
         self.instance = instance
         self.node = node
-        self.config = metalsmith.InstanceConfig(ssh_keys=ssh_keys)
+        self.config = instance_config.CloudInitConfig(ssh_keys=ssh_keys)
         self.config.add_user(ssh_user_name, admin=True, sudo=True)
-        self.default_image = default_image
         self.default_network = default_network
         self.default_root_size = default_root_size
 
     def run(self, context):
         try:
-            _validate_instances([self.instance],
-                                default_image=self.default_image)
+            _validate_instances([self.instance])
         except Exception as exc:
             LOG.error('Failed to validate the request. %s', exc)
             return actions.Result(error=six.text_type(exc))
@@ -306,9 +322,12 @@ class UndeployInstanceAction(base.TripleOAction):
 
     def run(self, context):
         provisioner = _provisioner(context)
-
+        if isinstance(self.instance, dict):
+            inst = self.instance['hostname']
+        else:
+            inst = self.instance
         try:
-            instance = provisioner.show_instance(self.instance)
+            instance = provisioner.show_instance(inst)
         except Exception:
             LOG.warning('Cannot get instance %s, assuming already deleted',
                         self.instance)
@@ -322,12 +341,26 @@ class UndeployInstanceAction(base.TripleOAction):
 class ExpandRolesAction(base.TripleOAction):
     """Convert a baremetal_deployment file to list of instances."""
 
-    def __init__(self, roles, stackname='overcloud'):
+    def __init__(self, roles, stackname='overcloud',
+                 default_image='overcloud-full', provisioned=True):
         super(ExpandRolesAction, self).__init__()
         self.roles = roles
         self.stackname = stackname
+        self.default_image = default_image
+        self.provisioned = provisioned
 
     def run(self, context):
+        for role in self.roles:
+            for inst in role.get('instances', []):
+                # Set the default image so that the
+                # source validation can succeed.
+                inst.setdefault('image', {'href': self.default_image})
+
+                # Set the default hostname now for duplicate hostname
+                # detection during validation
+                if 'hostname' not in inst and 'name' in inst:
+                    inst['hostname'] = inst['name']
+
         try:
             _validate_roles(self.roles, stackname=self.stackname)
         except Exception as exc:
@@ -339,44 +372,107 @@ class ExpandRolesAction(base.TripleOAction):
         parameter_defaults = {'HostnameMap': hostname_map}
         for role in self.roles:
             name = role['name']
-            hostname_format = role.get('hostname_format')
-            if not hostname_format:
-                hostname_format = '%stackname%-{}-%index%'.format(
-                    'novacompute' if name == 'Compute' else name.lower())
+            hostname_format = _hostname_format(
+                role.get('hostname_format'), name)
+            count = role.get('count', 1)
+            unprovisioned_indexes = []
+
+            # build a map of all potential generated names
+            # with the index number which generates the name
+            potential_gen_names = {}
+            for index in range(count + len(role.get('instances', []))):
+                potential_gen_names[_build_hostname(
+                    hostname_format, index, self.stackname)] = index
+
+            # build a list of instances from the specified
+            # instances list
+            role_instances = []
+            for instance in role.get('instances', []):
+                inst = {}
+                inst.update(role.get('defaults', {}))
+                inst.update(instance)
+                inst.setdefault('image', {'image': self.default_image})
+
+                if 'name' in inst and 'hostname' not in inst:
+                    inst['hostname'] = inst['name']
+
+                # create a hostname map entry now if the specified hostname
+                # is a valid generated name
+                if inst.get('hostname') in potential_gen_names:
+                    hostname_map[inst['hostname']] = inst['hostname']
+
+                role_instances.append(inst)
+
+            # add generated instance entries until the desired count of
+            # provisioned instances is reached
+            while len([i for i in role_instances
+                       if i.get('provisioned', True)]) < count:
+                inst = {}
+                inst.update(role.get('defaults', {}))
+                inst.setdefault('image', {'href': self.default_image})
+                role_instances.append(inst)
+
             # NOTE(dtantsur): our hostname format may differ from THT defaults,
             # so override it in the resulting environment
             parameter_defaults['%sDeployedServerHostnameFormat' % name] = (
                 hostname_format)
 
-            if 'instances' in role:
-                parameter_defaults['%sDeployedServerCount' % name] = len(
-                    role['instances'])
-                # TODO(dtantsur): ordering-dependent logic here, can we do
-                # better?
-                for index, instance in enumerate(role['instances']):
-                    gen_name = (hostname_format.replace('%index%', str(index))
-                                .replace('%stackname%', self.stackname))
-                    if 'hostname' not in instance:
-                        instance['hostname'] = gen_name
-                    hostname_map[gen_name] = instance['hostname']
-                    instances.append(instance)
-            else:
-                count = role.get('count', 1)
-                parameter_defaults['%sDeployedServerCount' % name] = count
-                if not count:
-                    continue
+            # ensure each instance has a unique non-empty hostname
+            # and a hostname map entry. Also build a list of indexes
+            # for unprovisioned instances
+            index = 0
+            for inst in role_instances:
+                provisioned = inst.get('provisioned', True)
+                gen_name = None
+                hostname = inst.get('hostname')
 
-                for index in range(count):
-                    hostname = (hostname_format.replace('%index%', str(index))
-                                .replace('%stackname%', self.stackname))
-                    inst = {'hostname': hostname}
-                    if 'profile' in role:
-                        inst['profile'] = role['profile']
+                if hostname not in hostname_map:
+                    while (not gen_name
+                           or gen_name in hostname_map):
+                        gen_name = _build_hostname(
+                            hostname_format, index, self.stackname)
+                        index += 1
+                    inst.setdefault('hostname', gen_name)
+                    hostname = inst.get('hostname')
+                    hostname_map[gen_name] = inst['hostname']
+
+                if not provisioned:
+                    if gen_name:
+                        unprovisioned_indexes.append(
+                            potential_gen_names[gen_name])
+                    elif hostname in potential_gen_names:
+                        unprovisioned_indexes.append(
+                            potential_gen_names[hostname])
+
+            if unprovisioned_indexes:
+                parameter_defaults['%sRemovalPolicies' % name] = [{
+                    'resource_list': unprovisioned_indexes
+                }]
+
+            provisioned_count = 0
+            for inst in role_instances:
+                provisioned = inst.pop('provisioned', True)
+
+                if provisioned:
+                    provisioned_count += 1
+
+                # Only add instances which match the desired provisioned state
+                if provisioned == self.provisioned:
                     instances.append(inst)
-                    hostname_map[hostname] = hostname
 
-        return {'instances': instances,
-                'environment': {'parameter_defaults': parameter_defaults}}
+            parameter_defaults['%sDeployedServerCount' % name] = (
+                provisioned_count)
+
+        try:
+            _validate_instances(instances)
+        except Exception as exc:
+            LOG.error('Failed to validate the request. %s', exc)
+            return actions.Result(error=six.text_type(exc))
+        if self.provisioned:
+            env = {'parameter_defaults': parameter_defaults}
+        else:
+            env = {}
+        return {'instances': instances, 'environment': env}
 
 
 class PopulateEnvironmentAction(base.TripleOAction):
@@ -435,16 +531,8 @@ class PopulateEnvironmentAction(base.TripleOAction):
         return self.environment
 
 
-def _validate_instances(instances, default_image='overcloud-full'):
-    for inst in instances:
-        if inst.get('name') and not inst.get('hostname'):
-            inst['hostname'] = inst['name']
-
-        # Set the default image so that the source validation can succeed.
-        inst.setdefault('image', default_image)
-
-    jsonschema.validate(instances, _INSTANCES_INPUT_SCHEMA)
-
+def _validate_instances(instances):
+    jsonschema.validate(instances, _INSTANCES_SCHEMA)
     hostnames = set()
     names = set()
     for inst in instances:
@@ -468,17 +556,26 @@ def _validate_roles(roles, stackname='overcloud'):
     jsonschema.validate(roles, _ROLES_INPUT_SCHEMA)
 
     for item in roles:
-        if 'count' in item and 'instances' in item:
-            raise ValueError("Count and instances cannot be provided together")
+        count = item.get('count', 1)
+        instances = item.get('instances', [])
+        instances = [i for i in instances if i.get('provisioned', True)]
+        name = item.get('name')
+        if len(instances) > count:
+            raise ValueError(
+                "%s: number of instance entries %s "
+                "cannot be greater than count %s" %
+                (name, len(instances), count)
+            )
 
+        defaults = item.get('defaults', {})
+        if 'hostname' in defaults:
+            raise ValueError("%s: cannot specify hostname in defaults"
+                             % name)
+        if 'name' in defaults:
+            raise ValueError("%s: cannot specify name in defaults"
+                             % name)
         if 'instances' in item:
-            for index, inst in enumerate(item['instances']):
-                if 'profile' not in inst and 'profile' in item:
-                    inst['profile'] = item['profile']
-
-            # NOTE(dtantsur): make a copy since _validate_instances modifies
-            # the original, and we don't need it at this stage.
-            _validate_instances(copy.deepcopy(item['instances']))
+            _validate_instances(item['instances'])
 
 
 def _release_nodes(provisioner, nodes):
@@ -493,10 +590,11 @@ def _release_nodes(provisioner, nodes):
 
 
 def _get_source(instance):
-    return sources.detect(image=instance.get('image'),
-                          kernel=instance.get('image_kernel'),
-                          ramdisk=instance.get('image_ramdisk'),
-                          checksum=instance.get('image_checksum'))
+    image = instance.get('image', {})
+    return sources.detect(image=image.get('href'),
+                          kernel=image.get('kernel'),
+                          ramdisk=image.get('ramdisk'),
+                          checksum=image.get('checksum'))
 
 
 def _instance_to_dict(connection, instance):
@@ -513,3 +611,16 @@ def _instance_to_dict(connection, instance):
             net_info['fixed_ips'].append({'ip_address': ip['ip_address']})
             net_info['subnets'].append(subnet.to_dict())
     return result
+
+
+def _hostname_format(hostname_format, role_name):
+    if not hostname_format:
+        hostname_format = '%stackname%-{}-%index%'.format(
+            'novacompute' if role_name == 'Compute' else role_name.lower())
+    return hostname_format
+
+
+def _build_hostname(hostname_format, index, stack):
+    gen_name = hostname_format.replace('%index%', str(index))
+    gen_name = gen_name.replace('%stackname%', stack)
+    return gen_name
