@@ -304,7 +304,7 @@ class TripleoInventory(object):
             'ceph_nfs': 'nfss',
             'ceph_client': 'clients',
             'ceph_rbdmirror': 'rbdmirrors',
-            'ceph_grafana-server': 'grafana-server'
+            'ceph_grafana': 'grafana-server'
         }
         # add a ceph-ansible compatible group to the inventory
         # which has the same roles. E.g. if the inventory has
@@ -319,14 +319,15 @@ class TripleoInventory(object):
             service_children = [role for role in roles
                                 if ret.get(role) is not None]
             if service_children:
-                ret[service.lower()] = {
+                svc_host = service.lower()
+                ret[svc_host] = {
                     'children': self._hosts(service_children),
                     'vars': {
                         'ansible_ssh_user': self.ansible_ssh_user
                     }
                 }
                 if self.ansible_python_interpreter:
-                    ret[role]['vars']['ansible_python_interpreter'] = \
+                    ret[svc_host]['vars']['ansible_python_interpreter'] = \
                         self.ansible_python_interpreter
 
         if not self.hosts_format_dict:
