@@ -26,10 +26,13 @@ class TestReserveNodes(base.TestCase):
 
     def test_success(self, mock_pr):
         instances = [
-            {'hostname': 'host1', 'profile': 'compute'},
+            {'hostname': 'host1', 'profile': 'compute',
+             'image': {'href': 'overcloud-full'}},
             {'hostname': 'host2', 'resource_class': 'compute',
-             'capabilities': {'answer': '42'}},
-            {'name': 'control-0', 'traits': ['CUSTOM_GPU']},
+             'capabilities': {'answer': '42'},
+             'image': {'href': 'overcloud-full'}},
+            {'name': 'control-0', 'traits': ['CUSTOM_GPU'],
+             'image': {'href': 'overcloud-full'}},
         ]
         action = baremetal_deploy.ReserveNodesAction(instances)
         result = action.run(mock.Mock())
@@ -50,8 +53,10 @@ class TestReserveNodes(base.TestCase):
 
     def test_missing_hostname(self, mock_pr):
         instances = [
-            {'hostname': 'host1'},
-            {'resource_class': 'compute', 'capabilities': {'answer': '42'}}
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
+            {'resource_class': 'compute', 'capabilities': {'answer': '42'},
+             'image': {'href': 'overcloud-full'}}
         ]
         action = baremetal_deploy.ReserveNodesAction(instances)
         result = action.run(mock.Mock())
@@ -72,10 +77,13 @@ class TestReserveNodes(base.TestCase):
     @mock.patch.object(baremetal_deploy.LOG, 'exception', autospec=True)
     def test_failure(self, mock_log, mock_pr):
         instances = [
-            {'hostname': 'host1'},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
             {'hostname': 'host2', 'resource_class': 'compute',
-             'capabilities': {'answer': '42'}},
-            {'hostname': 'host3'},
+             'capabilities': {'answer': '42'},
+             'image': {'href': 'overcloud-full'}},
+            {'hostname': 'host3',
+             'image': {'href': 'overcloud-full'}},
         ]
         success_node = mock.Mock(uuid='uuid1')
         mock_pr.return_value.reserve_node.side_effect = [
@@ -99,10 +107,13 @@ class TestReserveNodes(base.TestCase):
     @mock.patch.object(baremetal_deploy.LOG, 'exception', autospec=True)
     def test_failure_to_clean_up(self, mock_log, mock_pr):
         instances = [
-            {'hostname': 'host1'},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
             {'hostname': 'host2', 'resource_class': 'compute',
-             'capabilities': {'answer': '42'}},
-            {'hostname': 'host3'},
+             'capabilities': {'answer': '42'},
+             'image': {'href': 'overcloud-full'}},
+            {'hostname': 'host3',
+             'image': {'href': 'overcloud-full'}},
         ]
         success_node = mock.Mock(uuid='uuid1')
         mock_pr.return_value.reserve_node.side_effect = [
@@ -129,9 +140,10 @@ class TestReserveNodes(base.TestCase):
 
     def test_duplicate_hostname(self, mock_pr):
         instances = [
-            {'hostname': 'host1'},
-            # name is used as the default for hostname
-            {'name': 'host1'},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
         ]
         action = baremetal_deploy.ReserveNodesAction(instances)
         result = action.run(mock.Mock())
@@ -141,9 +153,11 @@ class TestReserveNodes(base.TestCase):
 
     def test_duplicate_name(self, mock_pr):
         instances = [
-            {'hostname': 'host1', 'name': 'node-1'},
+            {'hostname': 'host1', 'name': 'node-1',
+             'image': {'href': 'overcloud-full'}},
             # name is used as the default for hostname
-            {'name': 'node-1'},
+            {'name': 'node-1',
+             'image': {'href': 'overcloud-full'}},
         ]
         action = baremetal_deploy.ReserveNodesAction(instances)
         result = action.run(mock.Mock())
@@ -151,13 +165,42 @@ class TestReserveNodes(base.TestCase):
         self.assertIn('Node node-1 is requested more than once', result.error)
         self.assertFalse(mock_pr.return_value.reserve_node.called)
 
+    def test_hostname_format(self, mock_pr):
+        self.assertEqual(
+            '%stackname%-controller-%index%',
+            baremetal_deploy._hostname_format(None, 'Controller')
+        )
+        self.assertEqual(
+            '%stackname%-novacompute-%index%',
+            baremetal_deploy._hostname_format(None, 'Compute')
+        )
+        self.assertEqual(
+            'server-%index%',
+            baremetal_deploy._hostname_format('server-%index%', 'Compute')
+        )
+
+    def test_build_hostname(self, mock_pr):
+        self.assertEqual(
+            'overcloud-controller-2',
+            baremetal_deploy._build_hostname(
+                '%stackname%-controller-%index%', 2, 'overcloud'
+            )
+        )
+        self.assertEqual(
+            'server-2',
+            baremetal_deploy._build_hostname(
+                'server-%index%', 2, 'overcloud'
+            )
+        )
+
 
 @mock.patch.object(baremetal_deploy, '_provisioner', autospec=True)
 class TestDeployNode(base.TestCase):
 
     def test_success_defaults(self, mock_pr):
         action = baremetal_deploy.DeployNodeAction(
-            instance={'hostname': 'host1'},
+            instance={'hostname': 'host1',
+                      'image': {'href': 'overcloud-full'}},
             node='1234'
         )
         result = action.run(mock.Mock())
@@ -184,7 +227,11 @@ class TestDeployNode(base.TestCase):
 
     def test_success_with_name(self, mock_pr):
         action = baremetal_deploy.DeployNodeAction(
-            instance={'name': 'host1'},
+            instance={
+                'name': 'host1',
+                'hostname': 'host1',
+                'image': {'href': 'overcloud-full'}
+            },
             node='1234'
         )
         result = action.run(mock.Mock())
@@ -209,6 +256,7 @@ class TestDeployNode(base.TestCase):
     def test_success_advanced_nic(self, mock_pr):
         action = baremetal_deploy.DeployNodeAction(
             instance={'hostname': 'host1',
+                      'image': {'href': 'overcloud-full'},
                       'nics': [{'subnet': 'ctlplane-subnet'},
                                {'network': 'ctlplane',
                                 'fixed_ip': '10.0.0.2'}]},
@@ -238,7 +286,7 @@ class TestDeployNode(base.TestCase):
         pr = mock_pr.return_value
         action = baremetal_deploy.DeployNodeAction(
             instance={'hostname': 'host1',
-                      'image': 'overcloud-alt',
+                      'image': {'href': 'overcloud-alt'},
                       'nics': [{'port': 'abcd'}],
                       'root_size_gb': 100,
                       'swap_size_mb': 4096},
@@ -269,11 +317,15 @@ class TestDeployNode(base.TestCase):
 
     def test_success_http_partition_image(self, mock_pr):
         action = baremetal_deploy.DeployNodeAction(
-            instance={'hostname': 'host1',
-                      'image': 'https://example/image',
-                      'image_kernel': 'https://example/kernel',
-                      'image_ramdisk': 'https://example/ramdisk',
-                      'image_checksum': 'https://example/checksum'},
+            instance={
+                'hostname': 'host1',
+                'image': {
+                    'href': 'https://example/image',
+                    'kernel': 'https://example/kernel',
+                    'ramdisk': 'https://example/ramdisk',
+                    'checksum': 'https://example/checksum'
+                }
+            },
             node='1234'
         )
         result = action.run(mock.Mock())
@@ -303,11 +355,15 @@ class TestDeployNode(base.TestCase):
 
     def test_success_file_partition_image(self, mock_pr):
         action = baremetal_deploy.DeployNodeAction(
-            instance={'hostname': 'host1',
-                      'image': 'file:///var/lib/ironic/image',
-                      'image_kernel': 'file:///var/lib/ironic/kernel',
-                      'image_ramdisk': 'file:///var/lib/ironic/ramdisk',
-                      'image_checksum': 'abcd'},
+            instance={
+                'hostname': 'host1',
+                'image': {
+                    'href': 'file:///var/lib/ironic/image',
+                    'kernel': 'file:///var/lib/ironic/kernel',
+                    'ramdisk': 'file:///var/lib/ironic/ramdisk',
+                    'checksum': 'abcd'
+                }
+            },
             node='1234'
         )
         result = action.run(mock.Mock())
@@ -340,7 +396,8 @@ class TestDeployNode(base.TestCase):
     @mock.patch.object(baremetal_deploy.LOG, 'exception', autospec=True)
     def test_failure(self, mock_log, mock_pr):
         pr = mock_pr.return_value
-        instance = {'hostname': 'host1'}
+        instance = {'hostname': 'host1',
+                    'image': {'href': 'overcloud-full'}}
         action = baremetal_deploy.DeployNodeAction(
             instance=instance,
             node='1234'
@@ -366,7 +423,8 @@ class TestDeployNode(base.TestCase):
     @mock.patch.object(baremetal_deploy.LOG, 'exception', autospec=True)
     def test_failure_to_clean_up(self, mock_log, mock_pr):
         pr = mock_pr.return_value
-        instance = {'hostname': 'host1'}
+        instance = {'hostname': 'host1',
+                    'image': {'href': 'overcloud-full'}}
         action = baremetal_deploy.DeployNodeAction(
             instance=instance,
             node='1234'
@@ -399,10 +457,13 @@ class TestCheckExistingInstances(base.TestCase):
     def test_success(self, mock_pr):
         pr = mock_pr.return_value
         instances = [
-            {'hostname': 'host1'},
-            {'hostname': 'host3'},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
+            {'hostname': 'host3',
+             'image': {'href': 'overcloud-full'}},
             {'hostname': 'host2', 'resource_class': 'compute',
-             'capabilities': {'answer': '42'}}
+             'capabilities': {'answer': '42'},
+             'image': {'href': 'overcloud-full'}}
         ]
         existing = mock.MagicMock(hostname='host2')
         pr.show_instance.side_effect = [
@@ -415,8 +476,13 @@ class TestCheckExistingInstances(base.TestCase):
 
         self.assertEqual({
             'instances': [existing.to_dict.return_value],
-            'not_found': [{'hostname': 'host1', 'image': 'overcloud-full'},
-                          {'hostname': 'host3', 'image': 'overcloud-full'}]
+            'not_found': [{
+                'hostname': 'host1',
+                'image': {'href': 'overcloud-full'},
+            }, {
+                'hostname': 'host3',
+                'image': {'href': 'overcloud-full'},
+            }]
         }, result)
         pr.show_instance.assert_has_calls([
             mock.call(host) for host in ['host1', 'host3', 'host2']
@@ -424,7 +490,8 @@ class TestCheckExistingInstances(base.TestCase):
 
     def test_hostname_mismatch(self, mock_pr):
         instances = [
-            {'hostname': 'host1'},
+            {'hostname': 'host1',
+             'image': {'href': 'overcloud-full'}},
         ]
         mock_pr.return_value.show_instance.return_value.hostname = 'host2'
         action = baremetal_deploy.CheckExistingInstancesAction(instances)
@@ -435,7 +502,8 @@ class TestCheckExistingInstances(base.TestCase):
 
     def test_unexpected_error(self, mock_pr):
         instances = [
-            {'hostname': 'host%d' % i} for i in range(3)
+            {'image': {'href': 'overcloud-full'},
+             'hostname': 'host%d' % i} for i in range(3)
         ]
         mock_pr.return_value.show_instance.side_effect = RuntimeError('boom')
         action = baremetal_deploy.CheckExistingInstancesAction(instances)
@@ -455,7 +523,8 @@ class TestWaitForDeployment(base.TestCase):
         inst.to_dict.return_value = {'hostname': 'compute.cloud'}
 
         action = baremetal_deploy.WaitForDeploymentAction(
-            {'hostname': 'compute.cloud', 'uuid': 'uuid1'})
+            {'hostname': 'compute.cloud', 'uuid': 'uuid1',
+             'image': {'href': 'overcloud-full'}})
         result = action.run(mock.Mock())
 
         pr.wait_for_provisioning.assert_called_once_with(['uuid1'],
@@ -544,8 +613,10 @@ class TestExpandRoles(base.TestCase):
         result = action.run(mock.Mock())
         self.assertEqual(
             [
-                {'hostname': 'overcloud-novacompute-0'},
-                {'hostname': 'overcloud-controller-0'},
+                {'hostname': 'overcloud-novacompute-0',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'overcloud-controller-0',
+                 'image': {'href': 'overcloud-full'}},
             ],
             result['instances'])
         self.assertEqual(
@@ -564,21 +635,35 @@ class TestExpandRoles(base.TestCase):
             result['environment']['parameter_defaults'])
 
     def test_with_parameters(self):
-        roles = [
-            {'name': 'Compute', 'count': 2, 'profile': 'compute',
-             'hostname_format': 'compute-%index%.example.com'},
-            {'name': 'Controller', 'count': 3, 'profile': 'control',
-             'hostname_format': 'controller-%index%.example.com'},
-        ]
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute'
+            },
+            'hostname_format': 'compute-%index%.example.com'
+        }, {
+            'name': 'Controller',
+            'count': 3,
+            'defaults': {
+                'profile': 'control'
+            },
+            'hostname_format': 'controller-%index%.example.com'
+        }]
         action = baremetal_deploy.ExpandRolesAction(roles)
         result = action.run(mock.Mock())
         self.assertEqual(
             [
-                {'hostname': 'compute-0.example.com', 'profile': 'compute'},
-                {'hostname': 'compute-1.example.com', 'profile': 'compute'},
-                {'hostname': 'controller-0.example.com', 'profile': 'control'},
-                {'hostname': 'controller-1.example.com', 'profile': 'control'},
-                {'hostname': 'controller-2.example.com', 'profile': 'control'},
+                {'hostname': 'compute-0.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'compute-1.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'controller-0.example.com', 'profile': 'control',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'controller-1.example.com', 'profile': 'control',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'controller-2.example.com', 'profile': 'control',
+                 'image': {'href': 'overcloud-full'}},
             ],
             result['instances'])
         self.assertEqual(
@@ -600,29 +685,43 @@ class TestExpandRoles(base.TestCase):
             result['environment']['parameter_defaults'])
 
     def test_explicit_instances(self):
-        roles = [
-            {'name': 'Compute', 'count': 2, 'profile': 'compute',
-             'hostname_format': 'compute-%index%.example.com'},
-            {'name': 'Controller',
-             'profile': 'control',
-             'instances': [
-                 {'hostname': 'controller-X.example.com',
-                  'profile': 'control-X'},
-                 {'name': 'node-0', 'traits': ['CUSTOM_FOO'],
-                  'nics': [{'subnet': 'leaf-2'}]},
-             ]},
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute'
+            },
+            'hostname_format': 'compute-%index%.example.com'
+        }, {
+            'name': 'Controller',
+            'count': 2,
+            'defaults': {
+                'profile': 'control'
+            },
+            'instances': [{
+                'hostname': 'controller-X.example.com',
+                'profile': 'control-X'
+            }, {
+                'name': 'node-0',
+                'traits': ['CUSTOM_FOO'],
+                'nics': [{'subnet': 'leaf-2'}]},
+            ]},
         ]
         action = baremetal_deploy.ExpandRolesAction(roles)
         result = action.run(mock.Mock())
         self.assertEqual(
             [
-                {'hostname': 'compute-0.example.com', 'profile': 'compute'},
-                {'hostname': 'compute-1.example.com', 'profile': 'compute'},
+                {'hostname': 'compute-0.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'compute-1.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
                 {'hostname': 'controller-X.example.com',
+                 'image': {'href': 'overcloud-full'},
                  'profile': 'control-X'},
                 # Name provides the default for hostname later on.
                 {'name': 'node-0', 'profile': 'control',
-                 'hostname': 'overcloud-controller-1',
+                 'hostname': 'node-0',
+                 'image': {'href': 'overcloud-full'},
                  'traits': ['CUSTOM_FOO'], 'nics': [{'subnet': 'leaf-2'}]},
             ],
             result['instances'])
@@ -638,57 +737,439 @@ class TestExpandRoles(base.TestCase):
                     'compute-0.example.com': 'compute-0.example.com',
                     'compute-1.example.com': 'compute-1.example.com',
                     'overcloud-controller-0': 'controller-X.example.com',
-                    'overcloud-controller-1': 'overcloud-controller-1',
+                    'overcloud-controller-1': 'node-0',
                 }
             },
             result['environment']['parameter_defaults'])
 
     def test_count_with_instances(self):
-        roles = [
-            {'name': 'Compute', 'count': 2, 'profile': 'compute',
-             'hostname_format': 'compute-%index%.example.com'},
-            {'name': 'Controller',
-             'profile': 'control',
-             # Count makes no sense with instances and thus is disallowed.
-             'count': 3,
-             'instances': [
-                 {'hostname': 'controller-X.example.com',
-                  'profile': 'control-X'},
-                 {'name': 'node-0', 'traits': ['CUSTOM_FOO'],
-                  'nics': [{'subnet': 'leaf-2'}]},
-             ]},
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute',
+            },
+            'hostname_format': 'compute-%index%.example.com'
+        }, {
+            'name': 'Controller',
+            'defaults': {
+                'profile': 'control',
+            },
+            'count': 3,
+            'instances': [{
+                'hostname': 'controller-X.example.com',
+                'profile': 'control-X'
+            }, {
+                'name': 'node-0',
+                'traits': ['CUSTOM_FOO'],
+                'nics': [{'subnet': 'leaf-2'}]},
+            ]},
         ]
         action = baremetal_deploy.ExpandRolesAction(roles)
         result = action.run(mock.Mock())
-        self.assertIn("Count and instances cannot be provided together",
+        self.assertEqual([
+            {
+                'hostname': 'compute-0.example.com',
+                'profile': 'compute',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'compute-1.example.com',
+                'profile': 'compute',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'controller-X.example.com',
+                'profile': 'control-X',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'node-0',
+                'name': 'node-0',
+                'nics': [{'subnet': 'leaf-2'}],
+                'profile': 'control',
+                'traits': ['CUSTOM_FOO'],
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-2',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({
+            'ComputeDeployedServerCount': 2,
+            'ComputeDeployedServerHostnameFormat':
+                'compute-%index%.example.com',
+            'ControllerDeployedServerCount': 3,
+            'ControllerDeployedServerHostnameFormat':
+                '%stackname%-controller-%index%',
+            'HostnameMap': {
+                'compute-0.example.com': 'compute-0.example.com',
+                'compute-1.example.com': 'compute-1.example.com',
+                'overcloud-controller-0': 'controller-X.example.com',
+                'overcloud-controller-1': 'node-0',
+                'overcloud-controller-2': 'overcloud-controller-2'}
+            },
+            result['environment']['parameter_defaults'])
+
+    def test_unprovisioned(self):
+        roles = [{
+            'name': 'Controller',
+            'defaults': {
+                'profile': 'control',
+            },
+            'count': 2,
+            'instances': [{
+                'hostname': 'overcloud-controller-1',
+                'provisioned': False
+            }, {
+                'hostname': 'overcloud-controller-2',
+                'provisioned': False
+            }]
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'overcloud-controller-0',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-3',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({
+            'ControllerDeployedServerCount': 2,
+            'ControllerRemovalPolicies': [
+                {'resource_list': [1, 2]}
+            ],
+            'ControllerDeployedServerHostnameFormat':
+                '%stackname%-controller-%index%',
+            'HostnameMap': {
+                'overcloud-controller-0': 'overcloud-controller-0',
+                'overcloud-controller-1': 'overcloud-controller-1',
+                'overcloud-controller-2': 'overcloud-controller-2',
+                'overcloud-controller-3': 'overcloud-controller-3'}
+            },
+            result['environment']['parameter_defaults'])
+
+        # action for an unprovision workflow
+        action = baremetal_deploy.ExpandRolesAction(roles, provisioned=False)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'overcloud-controller-1',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-2',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({}, result['environment'])
+
+    def test_reprovisioned(self):
+        roles = [{
+            'name': 'Controller',
+            'defaults': {
+                'profile': 'control',
+            },
+            'count': 4,
+            'instances': [{
+                'hostname': 'overcloud-controller-1',
+                'provisioned': False
+            }, {
+                'hostname': 'overcloud-controller-2',
+                'provisioned': False
+            }]
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'overcloud-controller-0',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-3',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-4',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-5',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({
+            'ControllerDeployedServerCount': 4,
+            'ControllerRemovalPolicies': [
+                {'resource_list': [1, 2]}
+            ],
+            'ControllerDeployedServerHostnameFormat':
+                '%stackname%-controller-%index%',
+            'HostnameMap': {
+                'overcloud-controller-0': 'overcloud-controller-0',
+                'overcloud-controller-1': 'overcloud-controller-1',
+                'overcloud-controller-2': 'overcloud-controller-2',
+                'overcloud-controller-3': 'overcloud-controller-3',
+                'overcloud-controller-4': 'overcloud-controller-4',
+                'overcloud-controller-5': 'overcloud-controller-5'}
+            },
+            result['environment']['parameter_defaults'])
+
+        # action for an unprovision workflow
+        action = baremetal_deploy.ExpandRolesAction(roles, provisioned=False)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'overcloud-controller-1',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'overcloud-controller-2',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({}, result['environment'])
+
+    def test_unprovisioned_instances(self):
+        roles = [{
+            'name': 'Controller',
+            'defaults': {
+                'profile': 'control',
+            },
+            'count': 2,
+            'instances': [{
+                'name': 'node-0',
+                'hostname': 'controller-0'
+            }, {
+                'name': 'node-1',
+                'hostname': 'controller-1',
+                'provisioned': False
+            }, {
+                'name': 'node-2',
+                'hostname': 'controller-2',
+                'provisioned': False
+            }, {
+                'name': 'node-3',
+                'hostname': 'controller-3',
+                'provisioned': True
+            }]
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'controller-0',
+                'name': 'node-0',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'controller-3',
+                'name': 'node-3',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({
+            'ControllerDeployedServerCount': 2,
+            'ControllerRemovalPolicies': [
+                {'resource_list': [1, 2]}
+            ],
+            'ControllerDeployedServerHostnameFormat':
+                '%stackname%-controller-%index%',
+            'HostnameMap': {
+                'overcloud-controller-0': 'controller-0',
+                'overcloud-controller-1': 'controller-1',
+                'overcloud-controller-2': 'controller-2',
+                'overcloud-controller-3': 'controller-3'}
+            },
+            result['environment']['parameter_defaults'])
+
+        # action for an unprovision workflow
+        action = baremetal_deploy.ExpandRolesAction(roles, provisioned=False)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'controller-1',
+                'name': 'node-1',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'controller-2',
+                'name': 'node-2',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({}, result['environment'])
+
+    def test_unprovisioned_no_hostname(self):
+        roles = [{
+            'name': 'Controller',
+            'defaults': {
+                'profile': 'control',
+            },
+            'count': 2,
+            'instances': [{
+                'name': 'node-0',
+            }, {
+                'name': 'node-1',
+                'provisioned': False
+            }, {
+                'name': 'node-2',
+                'provisioned': False
+            }, {
+                'name': 'node-3',
+                'provisioned': True
+            }]
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'node-0',
+                'name': 'node-0',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'node-3',
+                'name': 'node-3',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({
+            'ControllerDeployedServerCount': 2,
+            'ControllerRemovalPolicies': [
+                {'resource_list': [1, 2]}
+            ],
+            'ControllerDeployedServerHostnameFormat':
+                '%stackname%-controller-%index%',
+            'HostnameMap': {
+                'overcloud-controller-0': 'node-0',
+                'overcloud-controller-1': 'node-1',
+                'overcloud-controller-2': 'node-2',
+                'overcloud-controller-3': 'node-3'}
+            },
+            result['environment']['parameter_defaults'])
+
+        # action for an unprovision workflow
+        action = baremetal_deploy.ExpandRolesAction(roles, provisioned=False)
+        result = action.run(mock.Mock())
+        self.assertEqual([
+            {
+                'hostname': 'node-1',
+                'name': 'node-1',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }, {
+                'hostname': 'node-2',
+                'name': 'node-2',
+                'profile': 'control',
+                'image': {'href': 'overcloud-full'}
+            }],
+            result['instances'])
+        self.assertEqual({}, result['environment'])
+
+    def test_name_in_defaults(self):
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute',
+                'name': 'compute-0'
+            }
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertIn('Compute: cannot specify name in defaults',
+                      result.error)
+
+    def test_hostname_in_defaults(self):
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute',
+                'hostname': 'compute-0'
+            }
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertIn('Compute: cannot specify hostname in defaults',
                       result.error)
 
     def test_instances_without_hostname(self):
-        roles = [
-            {'name': 'Compute', 'count': 2, 'profile': 'compute',
-             'hostname_format': 'compute-%index%.example.com'},
-            {'name': 'Controller',
-             'profile': 'control',
-             'instances': [
-                 {'profile': 'control-X'},  # missing hostname here
-                 {'name': 'node-0', 'traits': ['CUSTOM_FOO'],
-                  'nics': [{'subnet': 'leaf-2'}]},
-             ]},
+        roles = [{
+            'name': 'Compute',
+            'count': 2,
+            'defaults': {
+                'profile': 'compute'
+            },
+            'hostname_format': 'compute-%index%.example.com'
+        }, {
+            'name': 'Controller',
+            'count': 2,
+            'defaults': {
+                'profile': 'control'
+            },
+            'instances': [{
+                'profile': 'control-X'
+                # missing hostname here
+            }, {
+                'name': 'node-0',
+                'traits': ['CUSTOM_FOO'],
+                'nics': [{'subnet': 'leaf-2'}]},
+            ]},
         ]
         action = baremetal_deploy.ExpandRolesAction(roles)
         result = action.run(mock.Mock())
         self.assertEqual(
             [
-                {'hostname': 'compute-0.example.com', 'profile': 'compute'},
-                {'hostname': 'compute-1.example.com', 'profile': 'compute'},
-                {'hostname': 'overcloud-controller-0', 'profile': 'control-X'},
-                # Name provides the default for hostname later on.
+                {'hostname': 'compute-0.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'compute-1.example.com', 'profile': 'compute',
+                 'image': {'href': 'overcloud-full'}},
+                {'hostname': 'overcloud-controller-0', 'profile': 'control-X',
+                 'image': {'href': 'overcloud-full'}},
+                # Name provides the default for hostname
                 {'name': 'node-0', 'profile': 'control',
-                 'hostname': 'overcloud-controller-1',
+                 'hostname': 'node-0',
+                 'image': {'href': 'overcloud-full'},
                  'traits': ['CUSTOM_FOO'], 'nics': [{'subnet': 'leaf-2'}]},
             ],
             result['instances'])
 
+    def test_more_instances_than_count(self):
+        roles = [{
+            'name': 'Compute',
+            'count': 3,
+            'defaults': {
+                'profile': 'compute',
+                'name': 'compute-0'
+            },
+            'instances': [{
+                'name': 'node-0'
+            }, {
+                'name': 'node-1'
+            }, {
+                'name': 'node-2'
+            }, {
+                'name': 'node-3'
+            }]
+        }]
+        action = baremetal_deploy.ExpandRolesAction(roles)
+        result = action.run(mock.Mock())
+        self.assertIn('Compute: number of instance entries 4 '
+                      'cannot be greater than count 3',
+                      result.error)
+
 
 class TestPopulateEnvironment(base.TestCase):
 
