@@ -53,6 +53,8 @@ class TestImageUploadManager(base.TestCase):
         files.append('testfile')
         self.filelist = files
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._fetch_manifest')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -75,7 +77,8 @@ class TestImageUploadManager(base.TestCase):
                 'get_undercloud_registry', return_value='192.0.2.0:8787')
     def test_file_parsing(self, mock_gur, mockioctl, mockpath,
                           mock_images_match, mock_is_insecure, mock_inspect,
-                          mock_auth, mock_copy, mock_manifest):
+                          mock_auth, mock_copy, mock_manifest,
+                          check_status):
 
         mock_manifest.return_value = '{"layers": []}'
         mock_inspect.return_value = {}
@@ -110,6 +113,7 @@ class TestImageUploadManager(base.TestCase):
 
     @mock.patch('netifaces.ifaddresses')
     @mock.patch('netifaces.interfaces')
+    @mock.patch('netifaces.AF_INET6', 10)
     def test_get_undercloud_registry_ipv6(self, mock_interfaces,
                                           mock_addresses):
 
@@ -232,6 +236,54 @@ class TestImageUploadManager(base.TestCase):
         )
 
 
+class TestUploadTask(base.TestCase):
+    def test_basics(self):
+        obj = image_uploader.UploadTask(
+            image_name='foo:bar',
+            pull_source='docker.io/namespace',
+            push_destination='127.0.0.1:8787',
+            append_tag='baz',
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup=False,
+            multi_arch=False)
+        self.assertEqual(obj.repo, 'docker.io/namespace/foo')
+        self.assertEqual(obj.source_tag, 'bar')
+        self.assertEqual(obj.target_tag, 'barbaz')
+        self.assertEqual(obj.target_image_no_tag,
+                         '127.0.0.1:8787/namespace/foo')
+        self.assertEqual(obj.target_image,
+                         '127.0.0.1:8787/namespace/foo:barbaz')
+
+    def test_repo_pull_source_trailing_slash(self):
+        obj = image_uploader.UploadTask(
+            image_name='foo:bar',
+            pull_source='docker.io/namespace/',
+            push_destination='127.0.0.1:8787',
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup=False,
+            multi_arch=False)
+        self.assertEqual(obj.repo, 'docker.io/namespace/foo')
+
+    def test_repo_push_destination_trailing_slash(self):
+        obj = image_uploader.UploadTask(
+            image_name='foo:bar',
+            pull_source='docker.io/namespace',
+            push_destination='127.0.0.1:8787/',
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup=False,
+            multi_arch=False)
+        self.assertEqual(obj.target_image_no_tag,
+                         '127.0.0.1:8787/namespace/foo')
+
+
 class TestBaseImageUploader(base.TestCase):
 
     def setUp(self):
@@ -241,57 +293,56 @@ class TestBaseImageUploader(base.TestCase):
         self.uploader._inspect.retry.sleep = mock.Mock()
         self.requests = self.useFixture(rm_fixture.Fixture())
 
-    def test_is_insecure_registry_known(self):
+    @mock.patch.object(requests.Session, 'get', return_value=True)
+    def test_is_insecure_registry_known(self, mock_session):
         self.assertFalse(
             self.uploader.is_insecure_registry('docker.io'))
 
-    def test_is_insecure_registry_secure(self):
+    @mock.patch.object(requests.Session, 'get', return_value=True)
+    def test_is_insecure_registry_secure(self, mock_session):
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 1)
 
-    @mock.patch('requests.get')
-    def test_is_insecure_registry_bad_cert(self, mock_get):
-        mock_get.side_effect = [requests.exceptions.SSLError('ouch'), True]
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=[requests.exceptions.SSLError('err'), True])
+    def test_is_insecure_registry_bad_cert(self, mock_session):
         self.assertTrue(
             self.uploader.is_insecure_registry('bcert:8787'))
         self.assertTrue(
             self.uploader.is_insecure_registry('bcert:8787'))
         calls = [mock.call('https://bcert:8787/v2', timeout=30),
                  mock.call('https://bcert:8787/v2', timeout=30, verify=False)]
-        mock_get.assert_has_calls(calls)
-        self.assertEqual(mock_get.call_count, 2)
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 2)
 
-    def test_is_insecure_registry_timeout(self):
-        self.requests.get(
-            'https://192.0.2.0:8787/',
-            exc=requests.exceptions.ReadTimeout('ouch'))
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=requests.exceptions.ReadTimeout('ouch'))
+    def test_is_insecure_registry_timeout(self, mock_session):
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 1)
 
-    def test_is_insecure_registry_insecure(self):
-        self.requests.get(
-            'https://192.0.2.0:8787/v2',
-            exc=requests.exceptions.SSLError('ouch'))
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=requests.exceptions.SSLError('ouch'))
+    def test_is_insecure_registry_insecure(self, mock_session):
         self.assertTrue(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
         self.assertTrue(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30),
+                 mock.call('https://192.0.2.0:8787/v2', timeout=30,
+                           verify=False)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 2)
 
     @mock.patch('tripleo_common.image.image_uploader.'
                 'BaseImageUploader.authenticate')
@@ -542,7 +593,7 @@ class TestBaseImageUploader(base.TestCase):
         insecure_reg.add('registry-1.docker.io')
         secure_reg.add('192.0.2.1:8787')
         self.assertEqual(
-            'http://registry-1.docker.io/v2/t/nova-api/manifests/latest',
+            'https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
             build(url2, '/t/nova-api/manifests/latest')
         )
         self.assertEqual(
@@ -885,6 +936,8 @@ class TestSkopeoImageUploader(base.TestCase):
         self.uploader._copy.retry.sleep = mock.Mock()
         self.uploader._inspect.retry.sleep = mock.Mock()
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('os.environ')
     @mock.patch('subprocess.Popen')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -892,7 +945,7 @@ class TestSkopeoImageUploader(base.TestCase):
     @mock.patch('tripleo_common.image.image_uploader.'
                 'BaseImageUploader.authenticate')
     def test_upload_image(self, mock_auth, mock_inspect,
-                          mock_popen, mock_environ):
+                          mock_popen, mock_environ, check_status):
         mock_process = mock.Mock()
         mock_process.communicate.return_value = ('copy complete', '')
         mock_process.returncode = 0
@@ -1164,10 +1217,11 @@ class TestPythonImageUploader(base.TestCase):
         u._copy_layer_local_to_registry.retry.sleep = mock.Mock()
         u._copy_layer_registry_to_registry.retry.sleep = mock.Mock()
         u._copy_registry_to_registry.retry.sleep = mock.Mock()
-        u._copy_registry_to_local.retry.sleep = mock.Mock()
         u._copy_local_to_registry.retry.sleep = mock.Mock()
         self.requests = self.useFixture(rm_fixture.Fixture())
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1178,7 +1232,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_upload_image(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         target_session = mock.Mock()
         source_session = mock.Mock()
@@ -1261,6 +1315,8 @@ class TestPythonImageUploader(base.TestCase):
             multi_arch=False
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1271,7 +1327,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_authenticate_upload_image(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         self.uploader.registry_credentials = {
             'docker.io': {'my_username': 'my_password'},
@@ -1333,6 +1389,8 @@ class TestPythonImageUploader(base.TestCase):
             ),
         ])
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1343,7 +1401,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_insecure_registry(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
         target_session = mock.Mock()
         source_session = mock.Mock()
         authenticate.side_effect = [
@@ -1400,6 +1458,8 @@ class TestPythonImageUploader(base.TestCase):
             ),
         ])
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1410,7 +1470,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_upload_image_v1_manifest(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         target_session = mock.Mock()
         source_session = mock.Mock()
@@ -1489,6 +1549,8 @@ class TestPythonImageUploader(base.TestCase):
             multi_arch=False
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1508,7 +1570,8 @@ class TestPythonImageUploader(base.TestCase):
     def test_upload_image_modify(
             self, _copy_local_to_registry, run_modify_playbook,
             _copy_registry_to_local, _copy_registry_to_registry,
-            _cross_repo_mount, _fetch_manifest, _image_exists, authenticate):
+            _cross_repo_mount, _fetch_manifest, _image_exists, authenticate,
+            check_status):
 
         _image_exists.return_value = False
         target_session = mock.Mock()
@@ -1633,7 +1696,60 @@ class TestPythonImageUploader(base.TestCase):
             session=target_session
         )
 
-    def test_fetch_manifest(self):
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._detect_target_export')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.credentials_for_registry')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._copy_local_to_registry')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.authenticate')
+    def test_upload_image_local(self, authenticate, mock_copy, mock_creds,
+                                mock_detect):
+
+        mock_creds.return_value = (None, None)
+        target_session = mock.Mock()
+        authenticate.side_effect = [
+            target_session
+        ]
+
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
+        tag = 'latest'
+        push_destination = 'localhost:8787'
+        source_image = 'containers-storage:%s:%s' % (image, tag)
+        task = image_uploader.UploadTask(
+            image_name=source_image,
+            pull_source=None,
+            push_destination=push_destination,
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup='full',
+            multi_arch=False
+        )
+
+        self.assertEqual(
+            [],
+            self.uploader.upload_image(task)
+        )
+        source_url = urlparse(source_image)
+        target_url = urlparse('docker://localhost:8787/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+        authenticate.assert_has_calls([
+            mock.call(
+                target_url,
+                username=None,
+                password=None
+            )
+        ])
+        mock_detect.assert_called_once_with(target_url, target_session)
+        mock_copy.assert_called_once_with(source_url, target_url,
+                                          session=target_session)
+
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
+    def test_fetch_manifest(self, check_status):
         url = urlparse('docker://docker.io/t/nova-api:tripleo-current')
         manifest = '{"layers": []}'
         session = mock.Mock()
@@ -1652,8 +1768,9 @@ class TestPythonImageUploader(base.TestCase):
                           '.manifest.v2+json'
             }
         )
-
-    def test_upload_url(self):
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
+    def test_upload_url(self, check_status):
         # test with previous request
         previous_request = mock.Mock()
         previous_request.headers = {
@@ -1776,12 +1893,15 @@ class TestPythonImageUploader(base.TestCase):
             'docker'
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._upload_url')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.'
                 '_copy_layer_registry_to_registry')
-    def test_copy_registry_to_registry(self, _copy_layer, _upload_url):
+    def test_copy_registry_to_registry(self, _copy_layer, _upload_url,
+                                       check_status):
         source_url = urlparse('docker://docker.io/t/nova-api:latest')
         target_url = urlparse('docker://192.168.2.1:5000/t/nova-api:latest')
         _upload_url.return_value = 'https://192.168.2.1:5000/v2/upload'
@@ -1845,7 +1965,6 @@ class TestPythonImageUploader(base.TestCase):
                 params={'digest': 'sha256:1234'},
                 timeout=30
             ),
-            mock.call().raise_for_status(),
             mock.call(
                 'https://192.168.2.1:5000/v2/t/nova-api/manifests/latest',
                 data=mock.ANY,
@@ -1855,7 +1974,6 @@ class TestPythonImageUploader(base.TestCase):
                 },
                 timeout=30
             ),
-            mock.call().raise_for_status(),
         ])
         put_manifest = json.loads(
             target_session.put.call_args[1]['data'].decode('utf-8')
