@@ -16,7 +16,6 @@
 
 import mock
 import os
-import requests
 import six
 import sys
 import tempfile
@@ -237,7 +236,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'namespace': 'docker.io/tripleomaster',
                 'ceph_namespace': 'docker.io/ceph',
                 'ceph_image': 'daemon',
-                'ceph_tag': 'v4.0.0-stable-4.0-nautilus-centos-7-x86_64',
+                'ceph_tag': 'v4.0.1-stable-4.0-nautilus-centos-7-x86_64',
                 'ceph_grafana_namespace': 'docker.io/grafana',
                 'ceph_grafana_image': 'grafana',
                 'ceph_grafana_tag': 'latest',
@@ -253,53 +252,8 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'name_prefix': 'centos-binary-',
                 'name_suffix': '',
                 'tag': 'current-tripleo',
+                'rhel_containers': False,
                 'neutron_driver': 'ovn',
-                'openshift_namespace': 'docker.io/openshift',
-                'openshift_tag': 'v3.11.0',
-                'openshift_prefix': 'origin',
-                'openshift_cockpit_namespace': 'docker.io/cockpit',
-                'openshift_cockpit_image': 'kubernetes',
-                'openshift_cockpit_tag': 'latest',
-                'openshift_etcd_namespace': 'registry.fedoraproject.org'
-                '/latest',
-                'openshift_etcd_image': 'etcd',
-                'openshift_etcd_tag': 'latest',
-                'openshift_gluster_namespace': 'docker.io/gluster',
-                'openshift_gluster_image': 'gluster-centos',
-                'openshift_gluster_block_image': 'glusterblock-provisioner',
-                'openshift_gluster_tag': 'latest',
-                'openshift_heketi_namespace': 'docker.io/heketi',
-                'openshift_heketi_image': 'heketi',
-                'openshift_heketi_tag': 'latest',
-                'openshift_asb_namespace': 'docker.io/ansibleplaybookbundle',
-                'openshift_asb_tag': 'latest',
-                'openshift_cluster_monitoring_namespace': 'quay.io/coreos',
-                'openshift_cluster_monitoring_image':
-                    'cluster-monitoring-operator',
-                'openshift_cluster_monitoring_tag': 'v0.1.1',
-                'openshift_configmap_reload_namespace': 'quay.io/coreos',
-                'openshift_configmap_reload_image': 'configmap-reload',
-                'openshift_configmap_reload_tag': 'v0.0.1',
-                'openshift_prometheus_operator_namespace': 'quay.io/coreos',
-                'openshift_prometheus_operator_image': 'prometheus-operator',
-                'openshift_prometheus_operator_tag': 'v0.23.2',
-                'openshift_prometheus_config_reload_namespace':
-                    'quay.io/coreos',
-                'openshift_prometheus_config_reload_image':
-                    'prometheus-config-reloader',
-                'openshift_prometheus_config_reload_tag': 'v0.23.2',
-                'openshift_prometheus_tag': 'v2.3.2',
-                'openshift_prometheus_alertmanager_tag': 'v0.15.2',
-                'openshift_prometheus_node_exporter_tag': 'v0.16.0',
-                'openshift_oauth_proxy_tag': 'v1.1.0',
-                'openshift_kube_rbac_proxy_namespace': 'quay.io/coreos',
-                'openshift_kube_rbac_proxy_image': 'kube-rbac-proxy',
-                'openshift_kube_rbac_proxy_tag': 'v0.3.1',
-                'openshift_kube_state_metrics_namespace': 'quay.io/coreos',
-                'openshift_kube_state_metrics_image': 'kube-state-metrics',
-                'openshift_kube_state_metrics_tag': 'v1.3.1',
-                'openshift_grafana_namespace': 'docker.io/grafana',
-                'openshift_grafana_tag': '5.2.1',
             },
             builder.container_images_template_inputs()
         )
@@ -325,63 +279,8 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'name_prefix': 'prefix-',
                 'name_suffix': '-suffix',
                 'tag': 'master',
+                'rhel_containers': False,
                 'neutron_driver': 'ovn',
-                'openshift_namespace': 'docker.io/openshift3',
-                'openshift_tag': 'v3.11.0',
-                'openshift_prefix': 'ose',
-                'openshift_cockpit_namespace': 'docker.io/openshift-cockpit',
-                'openshift_cockpit_image': 'cockpit',
-                'openshift_cockpit_tag': 'cockpit-tag',
-                'openshift_etcd_namespace': 'registry.access.redhat.com/rhel7',
-                'openshift_etcd_image': 'openshift-etcd',
-                'openshift_etcd_tag': 'etcd-tag',
-                'openshift_gluster_namespace':
-                    'registry.access.redhat.com/rhgs3',
-                'openshift_gluster_image': 'rhgs-server-rhel7',
-                'openshift_gluster_block_image':
-                    'rhgs-gluster-block-prov-rhel7',
-                'openshift_gluster_tag': 'gluster-tag',
-                'openshift_heketi_namespace':
-                    'registry.access.redhat.com/rhgs3',
-                'openshift_heketi_image': 'rhgs-volmanager-rhel7',
-                'openshift_heketi_tag': 'heketi-tag',
-                'openshift_asb_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_asb_tag': 'v3.11',
-                'openshift_cluster_monitoring_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_cluster_monitoring_image':
-                    'ose-cluster-monitoring-operator',
-                'openshift_cluster_monitoring_tag': 'v3.11',
-                'openshift_configmap_reload_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_configmap_reload_image': 'ose-configmap-reloader',
-                'openshift_configmap_reload_tag': 'v3.11',
-                'openshift_prometheus_operator_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_prometheus_operator_image':
-                    'ose-prometheus-operator',
-                'openshift_prometheus_operator_tag': 'v3.11',
-                'openshift_prometheus_config_reload_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_prometheus_config_reload_image':
-                    'ose-prometheus-config-reloader',
-                'openshift_prometheus_config_reload_tag': 'v3.11',
-                'openshift_prometheus_tag': 'v3.11',
-                'openshift_prometheus_alertmanager_tag': 'v3.11',
-                'openshift_prometheus_node_exporter_tag': 'v3.11',
-                'openshift_oauth_proxy_tag': 'v3.11',
-                'openshift_kube_rbac_proxy_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_kube_rbac_proxy_image': 'ose-kube-rbac-proxy',
-                'openshift_kube_rbac_proxy_tag': 'v3.11',
-                'openshift_kube_state_metrics_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_kube_state_metrics_image': 'ose-kube-state-metrics',
-                'openshift_kube_state_metrics_tag': 'v3.11',
-                'openshift_grafana_namespace':
-                    'registry.access.redhat.com/openshift3',
-                'openshift_grafana_tag': 'v3.11',
             },
             builder.container_images_template_inputs(
                 namespace='192.0.2.0:5000/tripleomaster',
@@ -391,60 +290,8 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 name_prefix='prefix',
                 name_suffix='suffix',
                 tag='master',
+                rhel_containers=False,
                 neutron_driver='ovn',
-                openshift_namespace='docker.io/openshift3',
-                openshift_tag='v3.11.0',
-                openshift_prefix='ose',
-                openshift_cockpit_namespace='docker.io/openshift-cockpit',
-                openshift_cockpit_image='cockpit',
-                openshift_cockpit_tag='cockpit-tag',
-                openshift_etcd_namespace='registry.access.redhat.com/rhel7',
-                openshift_etcd_image='openshift-etcd',
-                openshift_etcd_tag='etcd-tag',
-                openshift_gluster_namespace='registry.access.redhat.com/rhgs3',
-                openshift_gluster_image='rhgs-server-rhel7',
-                openshift_gluster_block_image='rhgs-gluster-block-prov-rhel7',
-                openshift_gluster_tag='gluster-tag',
-                openshift_heketi_namespace='registry.access.redhat.com/rhgs3',
-                openshift_heketi_image='rhgs-volmanager-rhel7',
-                openshift_heketi_tag='heketi-tag',
-                openshift_asb_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_asb_tag='v3.11',
-                openshift_cluster_monitoring_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_cluster_monitoring_image='ose-'
-                    'cluster-monitoring-operator',
-                openshift_cluster_monitoring_tag='v3.11',
-                openshift_configmap_reload_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_configmap_reload_image='ose-configmap-reloader',
-                openshift_configmap_reload_tag='v3.11',
-                openshift_prometheus_operator_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_prometheus_operator_image='ose-'
-                    'prometheus-operator',
-                openshift_prometheus_operator_tag='v3.11',
-                openshift_prometheus_config_reload_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_prometheus_config_reload_image='ose-'
-                    'prometheus-config-reloader',
-                openshift_prometheus_config_reload_tag='v3.11',
-                openshift_prometheus_tag='v3.11',
-                openshift_prometheus_alertmanager_tag='v3.11',
-                openshift_prometheus_node_exporter_tag='v3.11',
-                openshift_oauth_proxy_tag='v3.11',
-                openshift_kube_rbac_proxy_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_kube_rbac_proxy_image='ose-kube-rbac-proxy',
-                openshift_kube_rbac_proxy_tag='v3.11',
-                openshift_kube_state_metrics_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_kube_state_metrics_image='ose-kube-state-metrics',
-                openshift_kube_state_metrics_tag='v3.11',
-                openshift_grafana_namespace='registry.'
-                    'access.redhat.com/openshift3',
-                openshift_grafana_tag='v3.11',
             )
         )
 
@@ -489,6 +336,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
         self.assertEqual(container_images, result)
 
     def _test_container_images_yaml_in_sync_helper(self, neutron_driver=None,
+                                                   rhel_containers=False,
                                                    remove_images=[]):
         '''Confirm overcloud_containers.tpl.yaml equals overcloud_containers.yaml
 
@@ -509,7 +357,8 @@ class TestKollaImageBuilderTemplate(base.TestCase):
             return entry
 
         result = tmpl_builder.container_images_from_template(
-            filter=ffunc, neutron_driver=neutron_driver)
+            filter=ffunc, neutron_driver=neutron_driver,
+            rhel_containers=rhel_containers)
 
         oc_yaml_file = os.path.join(files_dir, 'overcloud_containers.yaml')
         yaml_builder = kb.KollaImageBuilder([oc_yaml_file])
@@ -594,6 +443,33 @@ class TestKollaImageBuilderTemplate(base.TestCase):
         self._test_container_images_yaml_in_sync_helper(
             neutron_driver='ovn', remove_images=remove_images)
 
+    def test_container_images_yaml_in_sync_for_rhel(self):
+        remove_images = [
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-neutron-server:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-neutron-server-opendaylight:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-opendaylight:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-fluentd:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-sensu-client:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-skydive-agent:current-tripleo'},
+            {'image_source': 'kolla',
+                'imagename': 'docker.io/tripleomaster/centos-binary'
+                             '-skydive-analyzer:current-tripleo'}]
+        self._test_container_images_yaml_in_sync_helper(
+            rhel_containers=True, neutron_driver='ovn',
+            remove_images=remove_images)
+
 
 class TestPrepare(base.TestCase):
 
@@ -606,37 +482,25 @@ class TestPrepare(base.TestCase):
             with open(imagefile.name, 'w') as f:
                 f.write(template_filedata)
 
-    @mock.patch('requests.get')
-    def test_detect_insecure_registry(self, mock_get):
+    @mock.patch.object(image_uploader.ImageUploadManager, 'uploader')
+    def test_detect_insecure_registry(self, mock_uploader):
+        mock_f = mock.MagicMock()
+        mock_f.is_insecure_registry.side_effect = [False, True]
+        mock_uploader.return_value = mock_f
         self.assertEqual(
             {},
             kb.detect_insecure_registries(
                 {'foo': 'docker.io/tripleo'}))
         self.assertEqual(
-            {},
+            {'DockerInsecureRegistryAddress': ['tripleo']},
             kb.detect_insecure_registries(
                 {'foo': 'tripleo'}))
 
-    @mock.patch('requests.get')
-    def test_detect_insecure_registry_readtimeout(self, mock_get):
-
-        mock_get.side_effect = requests.exceptions.ReadTimeout('ouch')
-        self.assertEqual(
-            {},
-            kb.detect_insecure_registries(
-                {'foo': '192.0.2.0:8787/tripleo'}))
-
-    @mock.patch('requests.get')
-    def test_detect_insecure_registry_sslerror(self, mock_get):
-        mock_get.side_effect = requests.exceptions.SSLError('ouch')
-        self.assertEqual(
-            {'DockerInsecureRegistryAddress': ['192.0.2.0:8787']},
-            kb.detect_insecure_registries(
-                {'foo': '192.0.2.0:8787/tripleo'}))
-
-    @mock.patch('requests.get')
-    def test_detect_insecure_registry_multi(self, mock_get):
-        mock_get.side_effect = requests.exceptions.SSLError('ouch')
+    @mock.patch.object(image_uploader.ImageUploadManager, 'uploader')
+    def test_detect_insecure_registry_multi(self, mock_uploader):
+        mock_f = mock.MagicMock()
+        mock_f.is_insecure_registry.return_value = True
+        mock_uploader.return_value = mock_f
         self.assertEqual(
             {'DockerInsecureRegistryAddress': [
                 '192.0.2.0:8787',
@@ -647,15 +511,17 @@ class TestPrepare(base.TestCase):
                 'baz': '192.0.2.1:8787/tripleo/baz',
             }))
 
-    @mock.patch('requests.get')
-    def test_prepare_noargs(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_noargs(self, mock_insecure):
         self.assertEqual(
             {},
             kb.container_images_prepare(template_file=TEMPLATE_PATH)
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_simple(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_simple(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
                 {'image_source': 'kolla',
@@ -682,8 +548,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_includes(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_includes(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
                 {'image_source': 'kolla',
@@ -707,8 +574,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_includes_excludes(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_includes_excludes(self, mock_insecure):
         # assert same result as includes only. includes trumps excludes
         self.assertEqual({
             'container_images.yaml': [
@@ -734,8 +602,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_push_dest(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_push_dest(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [{
                 'image_source': 'kolla',
@@ -767,9 +636,10 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
     @mock.patch('tripleo_common.image.image_uploader.get_undercloud_registry')
-    def test_prepare_push_dest_discover(self, mock_gur, mock_get):
+    def test_prepare_push_dest_discover(self, mock_gur, mock_insecure):
         mock_gur.return_value = '192.0.2.0:8787'
         self.assertEqual({
             'container_images.yaml': [{
@@ -802,8 +672,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_ceph(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_ceph(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [{
                 'image_source': 'ceph',
@@ -825,8 +696,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_neutron_driver_default(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_neutron_driver_default(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
                 {'image_source': 'kolla',
@@ -853,8 +725,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_neutron_driver_ovn(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_neutron_driver_ovn(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
                 {'image_source': 'kolla',
@@ -886,8 +759,9 @@ class TestPrepare(base.TestCase):
             )
         )
 
-    @mock.patch('requests.get')
-    def test_prepare_neutron_driver_odl(self, mock_get):
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_neutron_driver_odl(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
                 {'image_source': 'kolla',
@@ -1004,7 +878,6 @@ class TestPrepare(base.TestCase):
                 'OS::TripleO::Services::HeatApi',
                 'OS::TripleO::Services::NovaApi',
                 'OS::TripleO::Services::NovaCompute',
-                'OS::TripleO::Services::OpenShift::Master',
                 'OS::TripleO::Services::Kubernetes::Worker',
                 'OS::TripleO::Services::SkydiveAgent',
             ]),
@@ -1016,8 +889,6 @@ class TestPrepare(base.TestCase):
                     '/tht/docker/services/foo.yaml',
                     'OS::TripleO::Services::NovaCompute':
                     '/tht/docker/services/foo.yaml',
-                    'OS::TripleO::Services::OpenShift::Master': 'deployment' +
-                    '/openshift/openshift-master-baremetal-ansible.yaml',
                     'OS::TripleO::Services::Kubernetes::Worker':
                     'deployment' +
                     'kubernetes/kubernetes-worker-baremetal-ansible.yaml',
@@ -1034,7 +905,6 @@ class TestPrepare(base.TestCase):
                         'OS::TripleO::Services::HeatApi',
                         'OS::TripleO::Services::NeutronApi',
                         'OS::TripleO::Services::NovaApi',
-                        'OS::TripleO::Services::OpenShift::Master',
                         'OS::TripleO::Services::SkydiveAgent',
                         'OS::TripleO::Services::Noop'
                     ]
