@@ -288,6 +288,30 @@ class TestInventory(base.TestCase):
                     'container_cli': 'podman',
                     'ctlplane_vip': 'x.x.x.4',
                     'redis_vip': 'x.x.x.6'}},
+            'sa': {'children': ['Controller'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sb': {'children': ['Controller'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sd': {'children': ['Compute'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'se': {'children': ['Compute'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'ceph_client': {'children': ['Compute'],
+                            'vars': {'ansible_ssh_user': 'my-custom-admin',
+                                     'ansible_python_interpreter': 'foo'}},
+            'clients': {'children': ['Compute'],
+                        'vars': {'ansible_ssh_user': 'my-custom-admin',
+                                 'ansible_python_interpreter': 'foo'}},
+            'sg': {'children': ['CustomRole'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sh': {'children': ['CustomRole'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
             'Undercloud': {
                 'hosts': ['undercloud'],
                 'vars': {'ansible_connection': 'ssh',
