@@ -70,6 +70,9 @@ class TestConfig(base.TestCase):
                 elif config == 'step_config':
                     expected_calls += [call('/tmp/tht/%s/%s.pp' %
                                             (role, config))]
+                elif config == 'param_config':
+                    expected_calls += [call('/tmp/tht/%s/%s.json' %
+                                            (role, config))]
                 else:
                     expected_calls += [call('/tmp/tht/%s/%s.yaml' %
                                             (role, config))]
@@ -297,6 +300,21 @@ class TestConfig(base.TestCase):
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
                          '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'HostnameNetworkConfigMap',
+             'output_value': {}},
+            {'output_key': 'AnsibleHostVarsMap',
+             'output_value': {
+                 'Controller': {
+                     'overcloud-controller-0': {
+                         'uuid': 0,
+                         'my_var': 'foo'}},
+                 'Compute': {
+                     'overcloud-novacompute-0': {
+                         'uuid': 1},
+                     'overcloud-novacompute-1': {
+                         'uuid': 2},
+                     'overcloud-novacompute-2': {
+                         'uuid': 3}}}},
             {'output_key': 'RoleGroupVars',
              'output_value': {
                  'Controller': {
@@ -418,6 +436,8 @@ class TestConfig(base.TestCase):
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
                          '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'HostnameNetworkConfigMap',
+             'output_value': {}},
             {'output_key': 'RoleGroupVars',
              'output_value': {
                  'Controller': {
@@ -497,40 +517,6 @@ class TestConfig(base.TestCase):
                           self.config.download_config, stack, self.tmp_dir)
         mock_git_init.assert_called_once_with(self.tmp_dir)
 
-    @patch.object(ooo_config.Config, 'initialize_git_repo')
-    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
-    @patch('tripleo_common.utils.config.Config.get_deployment_data')
-    def test_config_download_warn_grandparent_resource_name(
-        self, mock_deployment_data, mock_deployment_resource_id,
-        mock_git_init):
-        heat = mock.MagicMock()
-        self.config = ooo_config.Config(heat)
-        stack = mock.MagicMock()
-        heat.stacks.get.return_value = stack
-        heat.resources.get.return_value = mock.MagicMock()
-
-        deployment_data, _ = self._get_config_data('config_data.yaml')
-
-        # Set the name of the deployment to an integer to trigger looking up
-        # the grandparent resource name
-        deployment_data[0].attributes['value']['name'] = 1
-        self.deployments = deployment_data
-
-        mock_deployment_data.return_value = deployment_data
-        mock_deployment_resource_id.side_effect = self._get_deployment_id
-
-        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
-        with warnings.catch_warnings(record=True) as w:
-            self.assertRaises(ValueError,
-                              self.config.download_config, stack, self.tmp_dir)
-            self.assertGreaterEqual(len(w), 1)
-            self.assertGreaterEqual(len([x for x in w
-                                         if "grandparent"
-                                         in str(x.message)]),
-                                    1)
-
-        mock_git_init.assert_called_once_with(self.tmp_dir)
-
     @patch.object(ooo_config.Config, 'initialize_git_repo')
     @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
     @patch('tripleo_common.utils.config.Config.get_config_dict')
@@ -565,6 +551,8 @@ class TestConfig(base.TestCase):
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
                          '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'HostnameNetworkConfigMap',
+             'output_value': {}},
             {'output_key': 'RoleGroupVars',
              'output_value': {
                  'Controller': {
@@ -642,6 +630,9 @@ class TestConfig(base.TestCase):
                 elif config == 'step_config':
                     expected_calls += [call('/tmp/tht/%s/%s.pp' %
                                             (role, config))]
+                elif config == 'param_config':
+                    expected_calls += [call('/tmp/tht/%s/%s.json' %
+                                            (role, config))]
                 else:
                     expected_calls += [call('/tmp/tht/%s/%s.yaml' %
                                             (role, config))]
@@ -701,8 +692,10 @@ class TestConfig(base.TestCase):
                      'max_fail_percentage': 15},
                  'Compute': {
                      'any_errors_fatal': True,
-                     'max_fail_percentage': 15},
-             }}]
+                     'max_fail_percentage': 15}}},
+            {'output_key': 'HostnameNetworkConfigMap',
+             'output_value': {}}
+            ]
         deployment_data, configs = \
             self._get_config_data('config_data.yaml')
         self.configs = configs
@@ -814,3 +807,42 @@ class TestConfig(base.TestCase):
         self.config = ooo_config.Config(heat)
         self.assertRaises(yaml.scanner.ScannerError,
                           self.config.validate_config, stack_config, yaml_file)
+
+    @patch('tripleo_common.utils.config.Config.get_network_config_data')
+    def test_render_network_config_empty_dict(self,
+                                              mock_get_network_config_data):
+        heat = mock.MagicMock()
+        heat.stacks.get.return_value = fakes.create_tht_stack()
+        config_mock = mock.MagicMock()
+        config_mock.config = {}
+        heat.software_configs.get.return_value = config_mock
+
+        self.config = ooo_config.Config(heat)
+        stack = mock.Mock()
+        server_roles = dict(Controller='controller')
+        mock_get_network_config_data.return_value = dict(Controller='config')
+        config_dir = '/tmp/tht'
+        self.config.render_network_config(stack, config_dir, server_roles)
+
+    @patch.object(ooo_config.Config, '_open_file')
+    @patch('tripleo_common.utils.config.Config.get_network_config_data')
+    def test_render_network_config(self,
+                                   mock_get_network_config_data,
+                                   mock_open):
+        heat = mock.MagicMock()
+        heat.stacks.get.return_value = fakes.create_tht_stack()
+        config_mock = mock.MagicMock()
+        config_mock.config = 'some config'
+        heat.software_configs.get.return_value = config_mock
+
+        self.config = ooo_config.Config(heat)
+        stack = mock.Mock()
+        server_roles = dict(Controller='controller')
+        mock_get_network_config_data.return_value = dict(Controller='config')
+        config_dir = '/tmp/tht'
+        self.config.render_network_config(stack, config_dir, server_roles)
+        self.assertEqual(2, mock_open.call_count)
+        self.assertEqual('/tmp/tht/controller/Controller/NetworkConfig',
+                         mock_open.call_args_list[0][0][0])
+        self.assertEqual('/tmp/tht/controller/NetworkConfig',
+                         mock_open.call_args_list[1][0][0])
